
/*
 Copyright (c) 2006, 2007, The Cytoscape Consortium (www.cytoscape.org)

 The Cytoscape Consortium is:
 - Institute for Systems Biology
 - University of California San Diego
 - Memorial Sloan-Kettering Cancer Center
 - Institut Pasteur
 - Agilent Technologies

 This library is free software; you can redistribute it and/or modify it
 under the terms of the GNU Lesser General Public License as published
 by the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 This library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
 MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
 documentation provided hereunder is on an "as is" basis, and the
 Institute for Systems Biology and the Whitehead Institute
 have no obligations to provide maintenance, support,
 updates, enhancements or modifications.  In no event shall the
 Institute for Systems Biology and the Whitehead Institute
 be liable to any party for direct, indirect, special,
 incidental or consequential damages, including lost profits, arising
 out of the use of this software and its documentation, even if the
 Institute for Systems Biology and the Whitehead Institute
 have been advised of the possibility of such damage.  See
 the GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation,
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

package cytoscape.ding;

import cytoscape.render.stateful.GraphLOD;
import ding.view.PrintLOD;


/**
 * CyGraphAllLOD is the level of detail (LOD) class used
 * for the "Show Graphics Details" menu item.  It should
 * be thought of as a "preview" for printing or exporting
 * a network to some high-resolution graphical format (PDF,
 * SVG, etc.).  Essentially, this extends PrintLOD, but to
 * preserve some efficiency we override the textAsShape method
 * to allow us to continue to use fonts.
 */
public class CyGraphAllLOD extends PrintLOD {
	/**
	 *  textAsShape is called to determine if the text
	 *  labels should be converted from fonts to text
	 *
	 * @param renderNodeCount the number of nodes
	 * @param renderEdgeCount the number of edges
	 *
	 * @return  true if text should be converted to shapes, false otherwise
	 */
	public boolean textAsShape(int renderNodeCount, int renderEdgeCount) {
		return false;
	}

}
